"""
几何变换工具 - 处理OBB和坐标转换的数学运算
"""

import numpy as np
import math
from typing import Tuple, List


def normalize_coordinates(corners: np.ndarray, image_width: int, image_height: int) -> np.ndarray:
    """
    将像素坐标归一化到 [0, 1] 范围
    
    Args:
        corners: 角点坐标 (4, 2)
        image_width: 图片宽度
        image_height: 图片高度
        
    Returns:
        np.ndarray: 归一化后的坐标 (4, 2)
    """
    normalized = corners.copy()
    normalized[:, 0] = normalized[:, 0] / image_width  # x坐标
    normalized[:, 1] = normalized[:, 1] / image_height  # y坐标
    
    # 确保坐标在 [0, 1] 范围内
    normalized = np.clip(normalized, 0.0, 1.0)
    
    return normalized


def denormalize_coordinates(corners: np.ndarray, image_width: int, image_height: int) -> np.ndarray:
    """
    将归一化坐标转换为像素坐标
    
    Args:
        corners: 归一化坐标 (4, 2)
        image_width: 图片宽度
        image_height: 图片高度
        
    Returns:
        np.ndarray: 像素坐标 (4, 2)
    """
    pixels = corners.copy()
    pixels[:, 0] = pixels[:, 0] * image_width  # x坐标
    pixels[:, 1] = pixels[:, 1] * image_height  # y坐标
    
    return pixels


def calculate_obb_parameters(corners: np.ndarray) -> Tuple[float, float, float, float, float]:
    """
    从四个角点计算OBB参数（中心点、宽高、角度）
    
    Args:
        corners: 四个角点坐标 (4, 2)，顺序：左上，右上，右下，左下
        
    Returns:
        Tuple: (x_center, y_center, width, height, angle_degrees)
    """
    # 计算中心点
    center_x = np.mean(corners[:, 0])
    center_y = np.mean(corners[:, 1])
    
    # 计算宽度和高度
    # 宽度：左上到右上的距离
    width = np.linalg.norm(corners[1] - corners[0])
    # 高度：左上到左下的距离  
    height = np.linalg.norm(corners[3] - corners[0])
    
    # 计算角度（从左上到右上的向量与x轴的夹角）
    edge_vector = corners[1] - corners[0]
    angle_radians = np.arctan2(edge_vector[1], edge_vector[0])
    angle_degrees = np.degrees(angle_radians)
    
    # 确保角度在 [-90, 90] 范围内
    if angle_degrees > 90:
        angle_degrees -= 180
    elif angle_degrees < -90:
        angle_degrees += 180
    
    return center_x, center_y, width, height, angle_degrees


def obb_to_corners(center_x: float, center_y: float, width: float, height: float, 
                   angle_degrees: float) -> np.ndarray:
    """
    从OBB参数计算四个角点坐标
    
    Args:
        center_x: 中心点x坐标
        center_y: 中心点y坐标
        width: 宽度
        height: 高度
        angle_degrees: 角度（度）
        
    Returns:
        np.ndarray: 四个角点坐标 (4, 2)，顺序：左上，右上，右下，左下
    """
    # 转换为弧度
    angle_radians = np.radians(angle_degrees)
    cos_angle = np.cos(angle_radians)
    sin_angle = np.sin(angle_radians)
    
    # 在局部坐标系中的四个角点（相对于中心点）
    half_width = width / 2
    half_height = height / 2
    
    local_corners = np.array([
        [-half_width, -half_height],  # 左上
        [half_width, -half_height],   # 右上
        [half_width, half_height],    # 右下
        [-half_width, half_height]    # 左下
    ])
    
    # 旋转变换矩阵
    rotation_matrix = np.array([
        [cos_angle, -sin_angle],
        [sin_angle, cos_angle]
    ])
    
    # 应用旋转变换
    rotated_corners = local_corners @ rotation_matrix.T
    
    # 平移到实际位置
    global_corners = rotated_corners + np.array([center_x, center_y])
    
    return global_corners


def rect_to_corners(x_min: float, y_min: float, x_max: float, y_max: float) -> np.ndarray:
    """
    将矩形坐标转换为四个角点
    
    Args:
        x_min: 左上角x坐标
        y_min: 左上角y坐标
        x_max: 右下角x坐标
        y_max: 右下角y坐标
        
    Returns:
        np.ndarray: 四个角点坐标 (4, 2)
    """
    return np.array([
        [x_min, y_min],  # 左上
        [x_max, y_min],  # 右上
        [x_max, y_max],  # 右下
        [x_min, y_max]   # 左下
    ])


def corners_to_rect(corners: np.ndarray) -> Tuple[float, float, float, float]:
    """
    将角点坐标转换为轴对齐的矩形边界框
    
    Args:
        corners: 角点坐标 (4, 2)
        
    Returns:
        Tuple: (x_min, y_min, x_max, y_max)
    """
    x_min = np.min(corners[:, 0])
    y_min = np.min(corners[:, 1])
    x_max = np.max(corners[:, 0])
    y_max = np.max(corners[:, 1])
    
    return x_min, y_min, x_max, y_max


def yolo_to_corners(x_center: float, y_center: float, width: float, height: float) -> np.ndarray:
    """
    将YOLO格式（中心点+宽高）转换为四个角点
    
    Args:
        x_center: 中心点x坐标（归一化）
        y_center: 中心点y坐标（归一化）
        width: 宽度（归一化）
        height: 高度（归一化）
        
    Returns:
        np.ndarray: 四个角点坐标 (4, 2)
    """
    half_width = width / 2
    half_height = height / 2
    
    return np.array([
        [x_center - half_width, y_center - half_height],  # 左上
        [x_center + half_width, y_center - half_height],  # 右上
        [x_center + half_width, y_center + half_height],  # 右下
        [x_center - half_width, y_center + half_height]   # 左下
    ])


def corners_to_yolo(corners: np.ndarray) -> Tuple[float, float, float, float]:
    """
    将角点坐标转换为YOLO格式（中心点+宽高）
    
    Args:
        corners: 角点坐标 (4, 2)
        
    Returns:
        Tuple: (x_center, y_center, width, height)
    """
    x_min, y_min, x_max, y_max = corners_to_rect(corners)
    
    x_center = (x_min + x_max) / 2
    y_center = (y_min + y_max) / 2
    width = x_max - x_min
    height = y_max - y_min
    
    return x_center, y_center, width, height 


"""
基础格式抽象类 - 定义所有格式类的通用接口
"""

from abc import ABC, abstractmethod
from typing import List, Dict, Any, Optional
from pathlib import Path
import os

from .common_format import CommonFormat


class BaseFormat(ABC):
    """
    所有格式类的基类
    
    定义了格式转换的标准接口和通用方法
    """
    
    def __init__(self):
        """初始化格式类"""
        pass
    
    @property
    @abstractmethod
    def name(self) -> str:
        """格式名称"""
        pass
    
    @property
    @abstractmethod
    def file_extension(self) -> str:
        """文件扩展名"""
        pass
    
    @property
    @abstractmethod
    def description(self) -> str:
        """格式描述"""
        pass
    
    @abstractmethod
    def verify(self, file_path: str) -> bool:
        """
        验证文件是否符合该格式
        
        Args:
            file_path: 文件路径
            
        Returns:
            bool: 是否符合格式
        """
        pass
    
    @abstractmethod
    def _format2common(self, file_path: str, image_width: int, image_height: int,
                      class_names: Optional[List[str]] = None) -> CommonFormat:
        """
        将格式文件转换为中间格式（私有方法）
        
        Args:
            file_path: 输入文件路径
            image_width: 图片宽度
            image_height: 图片高度
            class_names: 类别名称列表（可选）
            
        Returns:
            CommonFormat: 中间格式对象
        """
        pass
    
    @abstractmethod
    def _common2format(self, common_data: CommonFormat, output_path: str) -> None:
        """
        将中间格式转换为该格式文件（私有方法）
        
        Args:
            common_data: 中间格式数据
            output_path: 输出文件路径
        """
        pass
    
    def format2commonSolo(self, file_path: str, image_width: int, image_height: int,
                         class_names: Optional[List[str]] = None) -> CommonFormat:
        """
        单文件转换：格式 -> 中间格式
        
        Args:
            file_path: 输入文件路径
            image_width: 图片宽度
            image_height: 图片高度
            class_names: 类别名称列表（可选）
            
        Returns:
            CommonFormat: 中间格式对象
        """
        
        return self._format2common(file_path, image_width, image_height, class_names)
    
    def common2formatSolo(self, common_data: CommonFormat, output_path: str) -> None:
        """
        单文件转换：中间格式 -> 格式
        
        Args:
            common_data: 中间格式数据
            output_path: 输出文件路径
        """
        # 确保输出目录存在
        output_dir = os.path.dirname(output_path)
        if output_dir:  # 只有当目录不为空时才创建
            os.makedirs(output_dir, exist_ok=True)
        
        self._common2format(common_data, output_path)
    
    def format2commonMulti(self, input_dir: str, image_width: int, image_height: int,
                          class_names: Optional[List[str]] = None) -> List[CommonFormat]:
        """
        多文件转换：格式 -> 中间格式
        
        Args:
            input_dir: 输入目录
            image_width: 图片宽度
            image_height: 图片高度
            class_names: 类别名称列表（可选）
            
        Returns:
            List[CommonFormat]: 中间格式对象列表
        """
        results = []
        input_path = Path(input_dir)
        
        # 查找所有符合扩展名的文件
        pattern = f"*{self.file_extension}"
        for file_path in input_path.glob(pattern):
            try:
                common_data = self._format2common(str(file_path), image_width, image_height, class_names)
                if common_data is not None:
                    common_data.image_filename = file_path.stem  # 保存文件名（不含扩展名）
                    results.append(common_data)
            except Exception as e:
                print(f"警告：处理文件 {file_path} 时出错: {e}")
        
        return results
    
    def common2formatMulti(self, common_data_list: List[CommonFormat], output_dir: str) -> None:
        """
        多文件转换：中间格式 -> 格式
        
        Args:
            common_data_list: 中间格式数据列表
            output_dir: 输出目录
        """
        # 确保输出目录存在
        os.makedirs(output_dir, exist_ok=True)
        
        for common_data in common_data_list:
            # 生成输出文件名
            if common_data.image_filename:
                output_filename = f"{common_data.image_filename}{self.file_extension}"
            else:
                output_filename = f"converted_{len(os.listdir(output_dir))}{self.file_extension}"
            
            output_path = os.path.join(output_dir, output_filename)
            
            try:
                self._common2format(common_data, output_path)
            except Exception as e:
                print(f"警告：生成文件 {output_path} 时出错: {e}")
    
    def _get_class_names(self, file_paths: List[str]) -> List[str]:
        """
        获取类别名称列表 - 通用实现
        
        Args:
            file_paths: 文件路径列表
            
        Returns:
            List[str]: 类别名称列表
        """
        if not file_paths:
            return []
        
        # 尝试从classes.txt文件读取（适用于YOLO系列格式）
        dir_path = os.path.dirname(file_paths[0])
        classes_file = os.path.join(dir_path, "classes.txt")
        
        if os.path.exists(classes_file) and os.path.isfile(classes_file):
            try:
                with open(classes_file, 'r', encoding='utf-8') as f:
                    class_names = [line.strip() for line in f.readlines() if line.strip()]
                    return class_names
            except Exception as e:
                print(f"警告：读取classes.txt文件失败: {e}")
        
        # 如果没有classes.txt文件，尝试从数据文件中解析
        return self._extract_class_names_from_files(file_paths)
    
    def _extract_class_names_from_files(self, file_paths: List[str]) -> List[str]:
        """
        从数据文件中提取类别名称 - 子类可重写此方法
        
        Args:
            file_paths: 文件路径列表
            
        Returns:
            List[str]: 类别名称列表
        """
        # 默认返回空列表，子类可根据需要重写
        return []
    
    def get_format_info(self) -> Dict[str, Any]:
        """
        获取格式信息
        
        Returns:
            Dict: 格式信息字典
        """
        return {
            'name': self.name,
            'file_extension': self.file_extension,
            'description': self.description
        } 


"""
YOLO-HBB 格式处理类

格式说明：
- 每行格式：class_id x_center y_center width height
- 坐标为归一化值（0-1之间）
- 这是传统的YOLO水平边界框格式
"""

import os
import numpy as np
from typing import List, Optional

from ..core.base_format import BaseFormat
from ..core.common_format import CommonFormat, BoundingBox
from ..core.geometry_utils import yolo_to_corners, corners_to_yolo


class YoloHBBFormat(BaseFormat):
    """YOLO-HBB 格式处理类"""
    
    @property
    def name(self) -> str:
        return "YOLO-HBB"
    
    @property
    def file_extension(self) -> str:
        return ".txt"
    
    @property
    def description(self) -> str:
        return "YOLO-HBB format: class_id x_center y_center width height (normalized coordinates)"
    
    def verify(self, file_path: str) -> bool:
        """
        验证文件是否符合YOLO-HBB格式
        
        Args:
            file_path: 文件路径
            
        Returns:
            bool: 是否符合格式
        """
        if not os.path.exists(file_path):
            return False
        
        if not file_path.endswith('.txt'):
            return False
        
        try:
            with open(file_path, 'r', encoding='utf-8') as f:
                lines = f.readlines()
            
            # 空文件也是有效的
            if not lines:
                return True
            
            for line in lines:
                line = line.strip()
                if not line:  # 跳过空行
                    continue
                
                parts = line.split()
                
                # 检查是否有5个部分（class_id + 4个参数）
                if len(parts) != 5:
                    return False
                
                # 检查class_id是否为整数
                try:
                    int(parts[0])
                except ValueError:
                    return False
                
                # 检查4个坐标参数是否为浮点数且在0-1范围内
                try:
                    coords = [float(x) for x in parts[1:]]
                    if not all(0 <= coord <= 1 for coord in coords):
                        return False
                except ValueError:
                    return False
            
            return True
            
        except Exception:
            return False
    
    def _format2common(self, file_path: str, image_width: int, image_height: int,
                      class_names: Optional[List[str]] = None) -> CommonFormat:
        """
        将YOLO-HBB格式转换为中间格式
        
        Args:
            file_path: 输入文件路径
            image_width: 图片宽度
            image_height: 图片高度
            class_names: 类别名称列表
            
        Returns:
            CommonFormat: 中间格式对象
        """
        if class_names is None:
            class_names = []
        
        bounding_boxes = []

        # 跳过 classes.txt
        if os.path.basename(file_path) == "classes.txt":
            return None
        
        with open(file_path, 'r', encoding='utf-8') as f:
            lines = f.readlines()
        
        for line in lines:
            line = line.strip()
            if not line:
                continue
            
            parts = line.split()
            if len(parts) != 5:
                continue
            
            class_id = int(parts[0])
            x_center = float(parts[1])
            y_center = float(parts[2])
            width = float(parts[3])
            height = float(parts[4])
            
            # 将YOLO格式转换为四个角点（归一化坐标）
            corners = yolo_to_corners(x_center, y_center, width, height)
            
            # 获取类别名称
            if class_id < len(class_names):
                class_name = class_names[class_id]
            else:
                class_name = f"class_{class_id}"
                # 扩展类别名称列表
                while len(class_names) <= class_id:
                    class_names.append(f"class_{len(class_names)}")
            
            # 创建边界框对象
            bbox = BoundingBox(
                class_name=class_name,
                corners=corners,
                class_id=class_id
            )
            
            bounding_boxes.append(bbox)
        
        return CommonFormat(
            image_width=image_width,
            image_height=image_height,
            bounding_boxes=bounding_boxes,
            class_names=class_names,
            image_filename=os.path.splitext(os.path.basename(file_path))[0]
        )
    
    def _common2format(self, common_data: CommonFormat, output_path: str) -> None:
        """
        将中间格式转换为YOLO-HBB格式
        
        Args:
            common_data: 中间格式数据
            output_path: 输出文件路径
        """
        lines = []
        
        for bbox in common_data.bounding_boxes:
            # 获取类别ID
            class_id = common_data.get_class_id(bbox.class_name)
            
            # 将角点转换为YOLO格式
            x_center, y_center, width, height = corners_to_yolo(bbox.corners)
            
            # 构建输出行
            line = f"{class_id} {x_center:.6f} {y_center:.6f} {width:.6f} {height:.6f}\n"
            lines.append(line)
        
        # 写入文件
        with open(output_path, 'w', encoding='utf-8') as f:
            f.writelines(lines) 

    def _generate_classes_txt(self, class_names: List[str], output_path: str) -> bool:
        """
        生成classes.txt文件
        """
        try:
            if os.path.exists(output_path) and os.path.isdir(output_path):
                dir_path = output_path
            else:
                dir_path = os.path.dirname(output_path)
            with open(os.path.join(dir_path, "classes.txt"), 'w', encoding='utf-8') as f:
                for class_name in class_names:
                    f.write(class_name + "\n")
            return True
        except Exception as e:
            print(f"Error generating classes.txt file: {e}")
            return False
        
    def common2formatSolo(self, common_data: CommonFormat, output_path: str) -> None:
        """
        将中间格式转换为YOLO-HBB格式
        """
        self._generate_classes_txt(common_data.class_names, output_path)
        super().common2formatSolo(common_data, output_path)
    
    def common2formatMulti(self, common_data_list: List[CommonFormat], output_path: str) -> None:
        """
        将中间格式转换为YOLO-HBB格式
        """
        if len(common_data_list) == 0:
            raise ValueError("common_data_list is empty")
        self._generate_classes_txt(common_data_list[0].class_names, output_path)
        super().common2formatMulti(common_data_list, output_path)


"""
格式管理器 - 管理所有支持的格式并执行转换
"""

from typing import List, Dict, Type, Optional
from .base_format import BaseFormat
from .common_format import CommonFormat
import os


class FormatManager:
    """
    格式管理器 - 集中管理所有支持的格式
    """
    
    def __init__(self):
        """初始化格式管理器"""
        self._formats: Dict[str, BaseFormat] = {}
        self._register_default_formats()
    
    def _register_default_formats(self) -> None:
        """注册默认支持的格式"""
        # 这里会在后续实现具体格式时导入并注册
        try:
            from ..formats.yolo_hbb import YoloHBBFormat
            self.register_format(YoloHBBFormat())
        except ImportError:
            pass
        
        try:
            from ..formats.yolo_obb import YoloOBBFormat
            self.register_format(YoloOBBFormat())
        except ImportError:
            pass
        
        try:
            from ..formats.labelimg_obb import LabelImgOBBFormat
            self.register_format(LabelImgOBBFormat())
        except ImportError:
            pass
        
        try:
            from ..formats.dota import DOTAFormat
            self.register_format(DOTAFormat())
        except ImportError:
            pass
        
        try:
            from ..formats.pascal_voc import PascalVOCFormat
            self.register_format(PascalVOCFormat())
        except ImportError:
            pass
    
    def register_format(self, format_instance: BaseFormat) -> None:
        """
        注册格式
        
        Args:
            format_instance: 格式实例
        """
        self._formats[format_instance.name] = format_instance
    
    def unregister_format(self, format_name: str) -> None:
        """
        注销格式
        
        Args:
            format_name: 格式名称
        """
        if format_name in self._formats:
            del self._formats[format_name]
    
    def get_format(self, format_name: str) -> BaseFormat:
        """
        获取格式实例
        
        Args:
            format_name: 格式名称
            
        Returns:
            BaseFormat: 格式实例
            
        Raises:
            ValueError: 如果格式不存在
        """
        if format_name not in self._formats:
            raise ValueError(f"Format '{format_name}' is not supported. "
                           f"Available formats: {list(self._formats.keys())}")
        return self._formats[format_name]
    
    def list_formats(self) -> List[str]:
        """
        列出所有支持的格式
        
        Returns:
            List[str]: 格式名称列表
        """
        return list(self._formats.keys())
    
    def get_format_info(self, format_name: str) -> Dict:
        """
        获取格式信息
        
        Args:
            format_name: 格式名称
            
        Returns:
            Dict: 格式信息
        """
        format_instance = self.get_format(format_name)
        return format_instance.get_format_info()
    
    def get_all_formats_info(self) -> Dict[str, Dict]:
        """
        获取所有格式的信息
        
        Returns:
            Dict: 所有格式信息
        """
        return {name: self.get_format_info(name) for name in self.list_formats()}
    
    def detect_format(self, file_path: str) -> Optional[str]:
        """
        自动检测文件格式
        
        Args:
            file_path: 文件路径
            
        Returns:
            Optional[str]: 检测到的格式名称，如果无法检测则返回None
        """
        for format_name, format_instance in self._formats.items():
            if format_instance.verify(file_path):
                return format_name
        return None
    
    def output_verbose(self, input_format: str, output_format: str, image_width: int, image_height: int, class_names: Optional[List[str]] = None) -> None:
        print(f"Input format: {input_format}")
        print(f"Output format: {output_format}")
        print(f"Image width: {image_width}")
        print(f"Image height: {image_height}")
        print(f"Class names: {class_names}")

    def convert_file(self, input_file: str, output_file: str, 
                    input_format: str, output_format: str,
                    image_width: int, image_height: int,
                    class_names: Optional[List[str]] = None, 
                    verbose: bool = False) -> None:
        """
        转换单个文件
        
        Args:
            input_file: 输入文件路径
            output_file: 输出文件路径
            input_format: 输入格式名称
            output_format: 输出格式名称
            image_width: 图片宽度
            image_height: 图片高度
            class_names: 类别名称列表（可选）
        """
        # 获取格式实例
        input_fmt = self.get_format(input_format)
        output_fmt = self.get_format(output_format)
        
        # 步骤1：输入格式 -> 中间格式
        if class_names is None:
            class_names = input_fmt._get_class_names([input_file])
        
        if verbose:
            self.output_verbose(input_format, output_format, image_width, image_height, class_names)
       
        
        common_data = input_fmt.format2commonSolo(input_file, image_width, image_height, class_names)
        
        # 步骤2：中间格式 -> 输出格式
        output_fmt.common2formatSolo(common_data, output_file)
    
    def convert_directory(self, input_dir: str, output_dir: str,
                         input_format: str, output_format: str,
                         image_width: int, image_height: int,
                         class_names: Optional[List[str]] = None,
                         verbose: bool = False) -> None:
        """
        转换整个目录
        
        Args:
            input_dir: 输入目录
            output_dir: 输出目录
            input_format: 输入格式名称
            output_format: 输出格式名称
            image_width: 图片宽度
            image_height: 图片高度
            class_names: 类别名称列表（可选）
        """
        # 获取格式实例
        input_fmt = self.get_format(input_format)
        output_fmt = self.get_format(output_format)
        # 步骤1：输入格式 -> 中间格式（批量）
        if class_names is None:
            if os.path.exists(input_dir) and os.path.isdir(input_dir):
                class_names = input_fmt._get_class_names([os.path.join(input_dir, f) for f in os.listdir(input_dir) if f.endswith(input_fmt.file_extension)])
            else:
                raise ValueError(f"Input directory {input_dir} is not a valid directory")
         
        if verbose:
            self.output_verbose(input_format, output_format, image_width, image_height, class_names)    
        
        common_data_list = input_fmt.format2commonMulti(input_dir, image_width, image_height, class_names)
        
        # 步骤2：中间格式 -> 输出格式（批量）
        output_fmt.common2formatMulti(common_data_list, output_dir)
    
    def is_format_supported(self, format_name: str) -> bool:
        """
        检查格式是否被支持
        
        Args:
            format_name: 格式名称
            
        Returns:
            bool: 是否支持
        """
        return format_name in self._formats


# 全局格式管理器实例
format_manager = FormatManager() 


"""
中间格式定义 - 统一的数据结构

所有格式都将转换为这个中间格式，然后再转换为目标格式
"""

from typing import List, Tuple, Optional, Dict, Any
from dataclasses import dataclass
import numpy as np


@dataclass
class BoundingBox:
    """
    边界框数据结构
    
    Attributes:
        class_name: 类别名称
        class_id: 类别ID（可选）
        corners: 四个角点坐标（归一化，顺序：左上，右上，右下，左下）
        confidence: 置信度（可选）
        difficulty: 难度级别（可选，DOTA格式使用）
    """
    class_name: str
    corners: np.ndarray  # shape: (4, 2) - 四个角点的 (x, y) 坐标
    class_id: Optional[int] = None
    confidence: Optional[float] = None
    difficulty: Optional[int] = None
    
    def __post_init__(self):
        """验证数据格式"""
        if self.corners.shape != (4, 2):
            raise ValueError("corners must be a (4, 2) numpy array")
        
        # 确保坐标是归一化的 (0-1之间)，允许微小的数值误差
        tolerance = 1e-6  # 允许微小的数值误差
        if np.any(self.corners < -tolerance) or np.any(self.corners > 1 + tolerance):
            raise ValueError("All coordinates must be normalized (0-1)")
        
        # 将坐标限制在[0, 1]范围内以消除数值误差
        self.corners = np.clip(self.corners, 0.0, 1.0)
    
    def to_dict(self) -> Dict[str, Any]:
        """转换为字典格式"""
        return {
            'class_name': self.class_name,
            'class_id': self.class_id,
            'corners': self.corners.tolist(),
            'confidence': self.confidence,
            'difficulty': self.difficulty
        }
    
    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> 'BoundingBox':
        """从字典创建对象"""
        return cls(
            class_name=data['class_name'],
            corners=np.array(data['corners']),
            class_id=data.get('class_id'),
            confidence=data.get('confidence'),
            difficulty=data.get('difficulty')
        )


@dataclass
class CommonFormat:
    """
    中间格式数据结构
    
    Attributes:
        image_width: 图片宽度（像素）
        image_height: 图片高度（像素）
        bounding_boxes: 边界框列表
        class_names: 类别名称列表（有序）
        image_filename: 图片文件名（可选）
    """
    image_width: int
    image_height: int
    bounding_boxes: List[BoundingBox]
    class_names: List[str]
    image_filename: Optional[str] = None
    
    def __post_init__(self):
        """验证数据"""
        if self.image_width <= 0 or self.image_height <= 0:
            raise ValueError("Image dimensions must be positive")
        
        # 验证边界框中的类别名称是否在类别列表中
        for bbox in self.bounding_boxes:
            if bbox.class_name not in self.class_names:
                raise ValueError(f"Class '{bbox.class_name}' not found in class_names")
    
    def add_bounding_box(self, bbox: BoundingBox) -> None:
        """添加边界框"""
        if bbox.class_name not in self.class_names:
            self.class_names.append(bbox.class_name)
        self.bounding_boxes.append(bbox)
    
    def get_class_id(self, class_name: str) -> int:
        """获取类别ID"""
        try:
            return self.class_names.index(class_name)
        except ValueError:
            raise ValueError(f"Class '{class_name}' not found")
    
    def get_class_name(self, class_id: int) -> str:
        """获取类别名称"""
        if 0 <= class_id < len(self.class_names):
            return self.class_names[class_id]
        raise ValueError(f"Class ID {class_id} out of range")
    
    def to_dict(self) -> Dict[str, Any]:
        """转换为字典格式"""
        return {
            'image_width': self.image_width,
            'image_height': self.image_height,
            'image_filename': self.image_filename,
            'class_names': self.class_names,
            'bounding_boxes': [bbox.to_dict() for bbox in self.bounding_boxes]
        }
    
    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> 'CommonFormat':
        """从字典创建对象"""
        return cls(
            image_width=data['image_width'],
            image_height=data['image_height'],
            class_names=data['class_names'],
            bounding_boxes=[BoundingBox.from_dict(bbox_data) for bbox_data in data['bounding_boxes']],
            image_filename=data.get('image_filename')
        ) 


"""
设置管理器 - 处理用户配置的保存和加载
"""

import os
import json
from typing import Dict, Any, Optional
from dataclasses import dataclass, asdict


@dataclass
class Settings:
    """用户设置数据类"""
    language: str = 'en'
    last_input_format: str = 'YOLO-OBB'
    last_output_format: str = 'LabelImg-OBB'
    last_input_path: str = ''
    last_output_path: str = ''
    last_image_width: int = 1920
    last_image_height: int = 1080
    last_class_names_file: str = ''
    window_width: int = 800
    window_height: int = 600
    window_x: int = 100
    window_y: int = 100
    theme: str = 'default'
    auto_detect_format: bool = True
    remember_last_paths: bool = True
    
    def to_dict(self) -> Dict[str, Any]:
        """转换为字典"""
        return asdict(self)
    
    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> 'Settings':
        """从字典创建设置对象"""
        # 过滤掉不存在的字段
        valid_fields = {field.name for field in cls.__dataclass_fields__.values()}
        filtered_data = {k: v for k, v in data.items() if k in valid_fields}
        return cls(**filtered_data)


class SettingsManager:
    """设置管理器"""
    
    def __init__(self):
        """初始化设置管理器"""
        self.settings_file = self._get_settings_file_path()
        self._settings = self._load_settings()
    
    def _get_settings_file_path(self) -> str:
        """获取设置文件路径"""
        # 获取用户主目录
        home_dir = os.path.expanduser('~')
        
        # 创建应用配置目录
        config_dir = os.path.join(home_dir, '.dataset_format_converter')
        if not os.path.exists(config_dir):
            os.makedirs(config_dir)
        
        return os.path.join(config_dir, 'settings.json')
    
    def _load_settings(self) -> Settings:
        """加载设置"""
        if os.path.exists(self.settings_file):
            try:
                with open(self.settings_file, 'r', encoding='utf-8') as f:
                    data = json.load(f)
                return Settings.from_dict(data)
            except Exception as e:
                print(f"警告：无法加载设置文件 {self.settings_file}: {e}")
                print("使用默认设置")
        
        # 返回默认设置
        return Settings()
    
    def save_settings(self) -> None:
        """保存设置到文件"""
        try:
            with open(self.settings_file, 'w', encoding='utf-8') as f:
                json.dump(self._settings.to_dict(), f, indent=2, ensure_ascii=False)
        except Exception as e:
            print(f"警告：无法保存设置文件 {self.settings_file}: {e}")
    
    def get_settings(self) -> Settings:
        """获取当前设置"""
        return self._settings
    
    def update_settings(self, **kwargs) -> None:
        """更新设置"""
        for key, value in kwargs.items():
            if hasattr(self._settings, key):
                setattr(self._settings, key, value)
            else:
                print(f"警告：未知的设置项 '{key}'")
    
    def reset_settings(self) -> None:
        """重置为默认设置"""
        self._settings = Settings()


# 全局设置管理器实例
_settings_manager = SettingsManager()


def get_settings() -> Settings:
    """获取当前设置"""
    return _settings_manager.get_settings()


def save_settings() -> None:
    """保存设置"""
    _settings_manager.save_settings()


def update_settings(**kwargs) -> None:
    """更新设置"""
    _settings_manager.update_settings(**kwargs)


def reset_settings() -> None:
    """重置设置"""
    _settings_manager.reset_settings() 


"""
核心模块 - 包含中间格式定义和基础功能
"""

from .common_format import CommonFormat, BoundingBox
from .format_manager import FormatManager
from .base_format import BaseFormat

__all__ = ['CommonFormat', 'BoundingBox', 'FormatManager', 'BaseFormat'] 


"""
DOTA 格式处理类

格式说明：
- 每行格式：x1 y1 x2 y2 x3 y3 x4 y4 class_name [difficulty]
- 坐标为像素值
- 四个点的坐标（顺序不定）
- 最后为类别名称，可选择性地包含难度级别
"""

import os
import numpy as np
from typing import List, Optional

from ..core.base_format import BaseFormat
from ..core.common_format import CommonFormat, BoundingBox
from ..core.geometry_utils import normalize_coordinates, denormalize_coordinates


class DOTAFormat(BaseFormat):
    """DOTA 格式处理类"""
    
    @property
    def name(self) -> str:
        return "DOTA"
    
    @property
    def file_extension(self) -> str:
        return ".txt"
    
    @property
    def description(self) -> str:
        return "DOTA format: x1 y1 x2 y2 x3 y3 x4 y4 class_name [difficulty] (pixel coordinates)"
    
    def verify(self, file_path: str) -> bool:
        """
        验证文件是否符合DOTA格式
        
        Args:
            file_path: 文件路径
            
        Returns:
            bool: 是否符合格式
        """
        if not os.path.exists(file_path):
            return False
        
        if not file_path.endswith('.txt'):
            return False
        
        try:
            with open(file_path, 'r', encoding='utf-8') as f:
                lines = f.readlines()
            
            # 空文件也是有效的
            if not lines:
                return True
            
            for line in lines:
                line = line.strip()
                if not line:  # 跳过空行
                    continue
                
                parts = line.split()
                
                # 检查是否至少有9个部分（8个坐标 + 类别名）
                if len(parts) < 9:
                    return False
                
                # 检查前8个是否为数字（坐标）
                try:
                    [float(x) for x in parts[:8]]
                except ValueError:
                    return False
                
                # 第9个必须是类别名（字符串）
                if not parts[8]:
                    return False
                
                # 如果有第10个，检查是否为数字（难度级别）
                if len(parts) > 9:
                    try:
                        int(parts[9])
                    except ValueError:
                        return False
            
            return True
            
        except Exception:
            return False
    
    def _format2common(self, file_path: str, image_width: int, image_height: int,
                      class_names: Optional[List[str]] = None) -> CommonFormat:
        """
        将DOTA格式转换为中间格式
        
        Args:
            file_path: 输入文件路径
            image_width: 图片宽度
            image_height: 图片高度
            class_names: 类别名称列表（将被更新）
            
        Returns:
            CommonFormat: 中间格式对象
        """
        if class_names is None:
            class_names = []
        
        bounding_boxes = []
        
        with open(file_path, 'r', encoding='utf-8') as f:
            lines = f.readlines()
        
        for line in lines:
            line = line.strip()
            if not line:
                continue
            
            parts = line.split()
            if len(parts) < 9:
                continue
            
            # 提取四个角点坐标（像素值）
            pixel_corners = np.array([
                [float(parts[0]), float(parts[1])],  # 第一个点
                [float(parts[2]), float(parts[3])],  # 第二个点
                [float(parts[4]), float(parts[5])],  # 第三个点
                [float(parts[6]), float(parts[7])]   # 第四个点
            ])
            
            # 转换为归一化坐标
            normalized_corners = normalize_coordinates(pixel_corners, image_width, image_height)
            
            # 提取类别名称
            class_name = parts[8]
            
            # 提取难度级别（可选）
            difficulty = None
            if len(parts) > 9:
                try:
                    difficulty = int(parts[9])
                except ValueError:
                    pass
            
            # 更新类别名称列表
            if class_name not in class_names:
                class_names.append(class_name)
            
            # 创建边界框对象
            bbox = BoundingBox(
                class_name=class_name,
                corners=normalized_corners,
                class_id=class_names.index(class_name),
                difficulty=difficulty
            )
            
            bounding_boxes.append(bbox)
        
        return CommonFormat(
            image_width=image_width,
            image_height=image_height,
            bounding_boxes=bounding_boxes,
            class_names=class_names,
            image_filename=os.path.splitext(os.path.basename(file_path))[0]
        )
    
    def _common2format(self, common_data: CommonFormat, output_path: str) -> None:
        """
        将中间格式转换为DOTA格式
        
        Args:
            common_data: 中间格式数据
            output_path: 输出文件路径
        """
        lines = []
        
        for bbox in common_data.bounding_boxes:
            # 将归一化坐标转换为像素坐标
            pixel_corners = denormalize_coordinates(
                bbox.corners,
                common_data.image_width,
                common_data.image_height
            )
            
            # 构建输出行
            line_parts = []
            
            # 添加8个坐标值
            for i in range(4):
                line_parts.extend([
                    f"{pixel_corners[i, 0]:.6f}",
                    f"{pixel_corners[i, 1]:.6f}"
                ])
            
            # 添加类别名称
            line_parts.append(bbox.class_name)
            
            # 添加难度级别（如果存在）
            if bbox.difficulty is not None:
                line_parts.append(str(bbox.difficulty))
            
            lines.append(" ".join(line_parts) + "\n")
        
        # 写入文件
        with open(output_path, 'w', encoding='utf-8') as f:
            f.writelines(lines) 

    def _extract_class_names_from_files(self, file_paths: List[str]) -> List[str]:
        """
        从DOTA格式文件中提取类别名称
        
        Args:
            file_paths: 文件路径列表
            
        Returns:
            List[str]: 类别名称列表
        """
        class_names = set()
        for file_path in file_paths:
            if not self.verify(file_path):
                continue
                
            try:
                with open(file_path, 'r', encoding='utf-8') as f:
                    lines = f.readlines()
                    
                for line in lines:
                    line = line.strip()
                    if not line:
                        continue
                    parts = line.split()
                    if len(parts) < 9:
                        continue
                    class_name = parts[8]
                    class_names.add(class_name)
            except Exception as e:
                print(f"警告：读取文件 {file_path} 时出错: {e}")
                
        return list(class_names)


"""
Dataset Format Converter - 多格式数据集标注转换工具

支持多种目标检测标注格式之间的相互转换：
- YOLO-HBB/YOLO
- YOLO-OBB (Ultralytics)
- LabelImg-OBB
- DOTA
- PASCAL VOC

主要功能：
- 多格式相互转换
- 统一的中间格式
- 批量处理
- 图形界面
- 命令行界面
- 多语言支持
"""

__version__ = "1.0.1.1"
__author__ = "Blake Zhu"
__email__ = "2112304124@mail2.gdut.edu.cn"

from .core.common_format import CommonFormat, BoundingBox
from .core.format_manager import FormatManager
from .formats.yolo_hbb import YoloHBBFormat
from .formats.yolo_obb import YoloOBBFormat
from .formats.labelimg_obb import LabelImgOBBFormat
from .formats.dota import DOTAFormat
from .formats.pascal_voc import PascalVOCFormat

__all__ = [
    'CommonFormat',
    'BoundingBox', 
    'FormatManager',
    'YoloHBBFormat',
    'YoloOBBFormat',
    'LabelImgOBBFormat',
    'DOTAFormat',
    'PascalVOCFormat'
] 


"""
格式模块 - 包含所有支持的数据格式实现
"""

from .yolo_hbb import YoloHBBFormat
from .yolo_obb import YoloOBBFormat
from .labelimg_obb import LabelImgOBBFormat
from .dota import DOTAFormat
from .pascal_voc import PascalVOCFormat

__all__ = [
    'YoloHBBFormat',
    'YoloOBBFormat', 
    'LabelImgOBBFormat',
    'DOTAFormat',
    'PascalVOCFormat'
]


"""
命令行界面主程序
"""

import argparse
import os
import sys
from typing import List, Optional

from ..core.format_manager import format_manager
from ..i18n.translation import t, set_language, get_available_languages
from ..config.settings import get_settings, update_settings, save_settings
from .. import __version__


def load_class_names(file_path: str) -> List[str]:
    """
    从文件加载类别名称
    
    Args:
        file_path: 类别名称文件路径
        
    Returns:
        List[str]: 类别名称列表
    """
    if not os.path.exists(file_path):
        return []
    
    try:
        with open(file_path, 'r', encoding='utf-8') as f:
            lines = f.readlines()
        
        # 过滤空行和注释
        class_names = []
        for line in lines:
            line = line.strip()
            if line and not line.startswith('#'):
                class_names.append(line)
        
        return class_names
    
    except Exception as e:
        print(f"{t('messages.error')}: {t('messages.file_not_found', file=file_path)}")
        return []


def interactive_mode():
    """交互模式"""
    print(f"\n=== {t('app.title')} ===")
    print(f"{t('app.description')}\n")
    
    # 显示支持的格式
    formats = format_manager.list_formats()
    print(f"{t('cli.input_format')}:")
    for i, fmt in enumerate(formats, 1):
        info = format_manager.get_format_info(fmt)
        print(f"  {i}. {fmt} - {info['description']}")
    
    # 选择输入格式
    while True:
        try:
            choice = input(f"\n{t('cli.input_format')} (1-{len(formats)}): ")
            if choice.lower() in ['q', 'quit', 'exit']:
                return
            
            index = int(choice) - 1
            if 0 <= index < len(formats):
                input_format = formats[index]
                break
            else:
                print(f"{t('messages.error')}: {t('messages.invalid_format', format=choice)}")
        except (ValueError, KeyboardInterrupt):
            print(f"\n{t('messages.error')}: {t('messages.invalid_format', format=choice)}")
            continue
    
    # 选择输出格式
    print(f"\n{t('cli.output_format')}:")
    for i, fmt in enumerate(formats, 1):
        info = format_manager.get_format_info(fmt)
        print(f"  {i}. {fmt} - {info['description']}")
    
    while True:
        try:
            choice = input(f"\n{t('cli.output_format')} (1-{len(formats)}): ")
            if choice.lower() in ['q', 'quit', 'exit']:
                return
            
            index = int(choice) - 1
            if 0 <= index < len(formats):
                output_format = formats[index]
                break
            else:
                print(f"{t('messages.error')}: {t('messages.invalid_format', format=choice)}")
        except (ValueError, KeyboardInterrupt):
            print(f"\n{t('messages.error')}: {t('messages.invalid_format', format=choice)}")
            continue
    
    # 输入文件/目录路径
    input_path = input(f"\n{t('cli.input')}: ").strip()
    if not input_path or not os.path.exists(input_path):
        print(f"{t('messages.error')}: {t('messages.file_not_found', file=input_path)}")
        return
    
    # 输出文件/目录路径
    output_path = input(f"{t('cli.output')}: ").strip()
    if not output_path:
        print(f"{t('messages.error')}: {t('cli.output')}")
        return
    
    # 图片尺寸
    try:
        width = int(input(f"{t('cli.width')} (1920): ") or "1920")
        height = int(input(f"{t('cli.height')} (1080): ") or "1080")
    except ValueError:
        print(f"{t('messages.error')}: {t('messages.invalid_dimensions')}")
        return
    
    # 类别名称文件（可选）
    class_names_file = input(f"{t('cli.classes')} ({t('cli.help')}): ").strip()
    class_names = None
    if class_names_file and os.path.exists(class_names_file):
        class_names = load_class_names(class_names_file)
    
    # 执行转换
    try:
        if os.path.isfile(input_path):
            print(f"\n{t('messages.processing_file', file=input_path)}")
            format_manager.convert_file(
                input_path, output_path, input_format, output_format,
                width, height, class_names
            )
        else:
            print(f"\n{t('messages.creating_output_dir', dir=output_path)}")
            format_manager.convert_directory(
                input_path, output_path, input_format, output_format,
                width, height, class_names
            )
        
        print(f"\n{t('messages.conversion_complete')}")
        
    except Exception as e:
        print(f"\n{t('messages.conversion_failed', error=str(e))}")


def main():
    """CLI主函数"""
    # 加载设置
    settings = get_settings()
    set_language(settings.language)
    
    parser = argparse.ArgumentParser(
        prog='dataset-format-converter',
        description=t('app.description'),
        formatter_class=argparse.RawDescriptionHelpFormatter
    )
    
    # 添加参数
    parser.add_argument(
        '--input', '-i',
        help=t('cli.input'),
        metavar='PATH'
    )
    
    parser.add_argument(
        '--output', '-o',
        help=t('cli.output'),
        metavar='PATH'
    )
    
    parser.add_argument(
        '--input-format', '-if',
        choices=format_manager.list_formats(),
        help=t('cli.input_format')
    )
    
    parser.add_argument(
        '--output-format', '-of',
        choices=format_manager.list_formats(),
        help=t('cli.output_format')
    )
    
    parser.add_argument(
        '--width', '-w',
        type=int,
        default=settings.last_image_width,
        help=f"{t('cli.width')} (默认: {settings.last_image_width})"
    )
    
    parser.add_argument(
        '--height',
        type=int,
        default=settings.last_image_height,
        help=f"{t('cli.height')} (默认: {settings.last_image_height})"
    )
    
    parser.add_argument(
        '--classes', '-c',
        help=t('cli.classes'),
        metavar='FILE'
    )
    
    parser.add_argument(
        '--language', '-l',
        choices=list(get_available_languages().keys()),
        help=t('gui.language')
    )
    
    parser.add_argument(
        '--version', '-v',
        action='version',
        version=f'%(prog)s {__version__}'
    )
    
    parser.add_argument(
        '--list-formats',
        action='store_true',
        help="列出所有支持的格式"
    )
    
    parser.add_argument(
        '--verbose',
        action='store_true',
        help="显示详细信息"
    )

    # 解析参数
    args = parser.parse_args()
    
    # 设置语言
    if args.language:
        set_language(args.language)
        update_settings(language=args.language)
        save_settings()
    
    # 列出格式
    if args.list_formats:
        print(f"\n{t('cli.options')}:")
        for fmt in format_manager.list_formats():
            info = format_manager.get_format_info(fmt)
            print(f"  {fmt:<15} - {info['description']}")
        return
    
    # 如果没有提供足够的参数，进入交互模式
    if not all([args.input, args.output, args.input_format, args.output_format]):
        interactive_mode()
        return
    
    # 验证输入
    if not os.path.exists(args.input):
        print(f"{t('messages.error')}: {t('messages.file_not_found', file=args.input)}")
        sys.exit(1)
    
    if args.width <= 0 or args.height <= 0:
        print(f"{t('messages.error')}: {t('messages.invalid_dimensions')}")
        sys.exit(1)
    
    # 加载类别名称
    class_names = None
    if args.classes:
        if os.path.exists(args.classes):
            class_names = load_class_names(args.classes)
        else:
            print(f"{t('messages.warning')}: {t('messages.file_not_found', file=args.classes)}")
    
    # 执行转换
    try:
        if os.path.isfile(args.input):
            print(f"{t('messages.processing_file', file=args.input)}")
            format_manager.convert_file(
                args.input, args.output, args.input_format, args.output_format,
                args.width, args.height, class_names, args.verbose
            )
        else:
            print(f"{t('messages.creating_output_dir', dir=args.output)}")
            format_manager.convert_directory(
                args.input, args.output, args.input_format, args.output_format,
                args.width, args.height, class_names, args.verbose
            )
        
        print(f"{t('messages.conversion_complete')}")
        
        # 更新设置
        update_settings(
            last_input_format=args.input_format,
            last_output_format=args.output_format,
            last_input_path=args.input,
            last_output_path=args.output,
            last_image_width=args.width,
            last_image_height=args.height,
            last_class_names_file=args.classes or ''
        )
        save_settings()
        
    except Exception as e:
        print(f"{t('messages.conversion_failed', error=str(e))}")
        sys.exit(1)


if __name__ == '__main__':
    main() 


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
GUI主窗口 - 提供图形化界面进行格式转换
"""

import os
import sys
import threading
from pathlib import Path
from typing import Optional, List

try:
    import tkinter as tk
    from tkinter import ttk, filedialog, messagebox, scrolledtext
    GUI_AVAILABLE = True
except ImportError:
    GUI_AVAILABLE = False

from ..core.format_manager import format_manager
from ..config.settings import get_settings, update_settings, save_settings
from ..i18n.translation import get_available_languages, set_language, t


class DatasetConverterGUI:
    """数据集格式转换器图形界面"""
    
    def __init__(self):
        """初始化GUI"""
        if not GUI_AVAILABLE:
            raise ImportError("tkinter模块不可用，无法启动图形界面")
        
        # 加载设置
        self.settings = get_settings()
        set_language(self.settings.language)
        
        # 初始化变量
        self.root = tk.Tk()
        self.current_class_names = []
        self.setup_window()
        self.setup_styles()
        self.create_widgets()
        
        # 在界面完全创建后加载设置
        self.root.after(100, self.load_settings)
        
        # 绑定关闭事件
        self.root.protocol("WM_DELETE_WINDOW", self.on_closing)
    
    def setup_window(self):
        """设置窗口属性"""
        self.root.title(t('app.title'))
        
        # 设置固定窗口大小
        window_width = 900
        window_height = 700
        self.root.geometry(f"{window_width}x{window_height}")
        
        # 锁定窗口大小，禁止调整
        self.root.resizable(False, False)
        
        # 居中显示窗口
        self.root.update_idletasks()
        x = (self.root.winfo_screenwidth() // 2) - (window_width // 2)
        y = (self.root.winfo_screenheight() // 2) - (window_height // 2)
        self.root.geometry(f"{window_width}x{window_height}+{x}+{y}")
        
        # 设置图标（如果有的话）
        try:
            # 这里可以设置应用图标
            pass
        except:
            pass
    
    def setup_styles(self):
        """设置样式"""
        style = ttk.Style()
        
        # 配置样式
        style.configure('Title.TLabel', font=('TkDefaultFont', 14, 'bold'), foreground='#2c3e50')
        style.configure('Header.TLabel', font=('TkDefaultFont', 10, 'bold'), foreground='#34495e')
        style.configure('Info.TLabel', font=('TkDefaultFont', 9), foreground='#7f8c8d')
        style.configure('Success.TLabel', font=('TkDefaultFont', 9), foreground='#27ae60')
        style.configure('Error.TLabel', font=('TkDefaultFont', 9), foreground='#e74c3c')
        style.configure('Primary.TButton', font=('TkDefaultFont', 11, 'bold'))
        
        # 配置Notebook样式
        style.configure('TNotebook.Tab', padding=[15, 10], font=('TkDefaultFont', 10))
        style.configure('TNotebook', tabposition='n')
        
        # 配置LabelFrame样式
        style.configure('TLabelframe.Label', font=('TkDefaultFont', 10, 'bold'), foreground='#2c3e50')
        style.configure('TLabelframe', borderwidth=2, relief='groove')
    
    def create_widgets(self):
        """创建界面组件"""
        # 配置根窗口网格
        self.root.columnconfigure(0, weight=1)
        self.root.rowconfigure(0, weight=1)
        
        # 创建主标题
        title_frame = ttk.Frame(self.root)
        title_frame.grid(row=0, column=0, sticky=(tk.W, tk.E), padx=25, pady=(20, 10))
        
        title_label = ttk.Label(title_frame, text=t('app.title'), style='Title.TLabel')
        title_label.pack(pady=(0, 3))
        
        subtitle_label = ttk.Label(title_frame, text=t('app.description'), style='Info.TLabel')
        subtitle_label.pack()
        
        # 添加分隔线
        separator = ttk.Separator(self.root, orient='horizontal')
        separator.grid(row=1, column=0, sticky=(tk.W, tk.E), padx=25, pady=(0, 15))
        
        # 创建Tab界面
        self.notebook = ttk.Notebook(self.root)
        self.notebook.grid(row=2, column=0, sticky=(tk.W, tk.E, tk.N, tk.S), padx=25, pady=(0, 15))
        self.root.rowconfigure(2, weight=1)
        
        # 创建各个Tab
        self.create_converter_tab()
        self.create_settings_tab()
        
        # 状态栏
        self.create_status_bar()
    
    def create_converter_tab(self):
        """创建转换器Tab"""
        # 创建转换器框架
        converter_frame = ttk.Frame(self.notebook)
        self.notebook.add(converter_frame, text=t('gui.converter'))
        
        # 创建滚动框架
        canvas = tk.Canvas(converter_frame)
        scrollbar = ttk.Scrollbar(converter_frame, orient="vertical", command=canvas.yview)
        scrollable_frame = ttk.Frame(canvas)
        
        scrollable_frame.bind(
            "<Configure>",
            lambda e: canvas.configure(scrollregion=canvas.bbox("all"))
        )
        
        canvas.create_window((0, 0), window=scrollable_frame, anchor="nw")
        canvas.configure(yscrollcommand=scrollbar.set)
        
        canvas.grid(row=0, column=0, sticky=(tk.W, tk.E, tk.N, tk.S))
        scrollbar.grid(row=0, column=1, sticky=(tk.N, tk.S))
        
        converter_frame.columnconfigure(0, weight=1)
        converter_frame.rowconfigure(0, weight=1)
        
        # 内容区域
        content_frame = ttk.Frame(scrollable_frame, padding="20")
        content_frame.grid(row=0, column=0, sticky=(tk.W, tk.E))
        content_frame.columnconfigure(0, weight=1)
        
        # 步骤1：选择格式
        self.create_format_selection_step(content_frame, 0)
        
        # 步骤2：选择输入
        self.create_input_selection_step(content_frame, 1)
        
        # 步骤3：选择输出
        self.create_output_selection_step(content_frame, 2)
        
        # 步骤4：图片尺寸
        self.create_dimensions_step(content_frame, 3)
        
        # 步骤5：类别确认
        self.create_classes_confirmation_step(content_frame, 4)
        
        # 转换区域
        self.create_conversion_step(content_frame, 5)
    
    def create_settings_tab(self):
        """创建设置Tab"""
        settings_frame = ttk.Frame(self.notebook, padding="20")
        self.notebook.add(settings_frame, text=t('gui.settings'))
        
        # 语言设置
        self.create_language_section(settings_frame)
        
        # 作者信息
        self.create_author_section(settings_frame)
        
        # 其他设置可以在这里添加
        
    def create_status_bar(self):
        """创建状态栏"""
        status_frame = ttk.Frame(self.root)
        status_frame.grid(row=3, column=0, sticky=(tk.W, tk.E), padx=25, pady=(0, 15))
        
        self.status_var = tk.StringVar(value=t('gui.ready'))
        status_label = ttk.Label(status_frame, textvariable=self.status_var, 
                               relief=tk.SUNKEN, anchor=tk.W, padding="8")
        status_label.pack(fill=tk.X)
        
        # 进度条
        self.progress_var = tk.DoubleVar()
        self.progress_bar = ttk.Progressbar(status_frame, variable=self.progress_var, 
                                          maximum=100, mode='determinate')
        self.progress_bar.pack(fill=tk.X, pady=(8, 0))
    
    def create_format_selection_step(self, parent, row):
        """步骤1：格式选择"""
        step_frame = ttk.LabelFrame(parent, text=f"📋 {t('gui.step1_formats')}", padding="20")
        step_frame.grid(row=row, column=0, sticky=(tk.W, tk.E), pady=(0, 20))
        step_frame.columnconfigure(1, weight=1)
        
        # 创建两列布局
        left_frame = ttk.Frame(step_frame)
        left_frame.grid(row=0, column=0, sticky=(tk.W, tk.E, tk.N, tk.S), padx=(0, 15))
        
        right_frame = ttk.Frame(step_frame)
        right_frame.grid(row=0, column=1, sticky=(tk.W, tk.E, tk.N, tk.S))
        
        step_frame.columnconfigure(0, weight=1)
        step_frame.columnconfigure(1, weight=1)
        
        # 左列：输入格式
        ttk.Label(left_frame, text=t('gui.input_format'), style='Header.TLabel').pack(anchor=tk.W, pady=(0, 8))
        
        self.input_format_var = tk.StringVar()
        formats_with_empty = [""] + format_manager.list_formats()
        input_format_combo = ttk.Combobox(left_frame, textvariable=self.input_format_var,
                                        values=formats_with_empty,
                                        state="readonly", width=28)
        input_format_combo.pack(anchor=tk.W, pady=(0, 10))
        input_format_combo.bind('<<ComboboxSelected>>', self.on_input_format_change)
        
        # 输入格式示例
        ttk.Label(left_frame, text="格式示例:", style='Info.TLabel').pack(anchor=tk.W, pady=(0, 5))
        self.input_format_example = scrolledtext.ScrolledText(left_frame, height=6, width=45, 
                                                             state='disabled', wrap=tk.WORD,
                                                             font=('Consolas', 9))
        self.input_format_example.pack(fill=tk.BOTH, expand=True)
        
        # 右列：输出格式
        ttk.Label(right_frame, text=t('gui.output_format'), style='Header.TLabel').pack(anchor=tk.W, pady=(0, 8))
        
        self.output_format_var = tk.StringVar()
        output_format_combo = ttk.Combobox(right_frame, textvariable=self.output_format_var,
                                         values=formats_with_empty,
                                         state="readonly", width=28)
        output_format_combo.pack(anchor=tk.W, pady=(0, 10))
        output_format_combo.bind('<<ComboboxSelected>>', self.on_output_format_change)
        
        # 输出格式示例
        ttk.Label(right_frame, text="格式示例:", style='Info.TLabel').pack(anchor=tk.W, pady=(0, 5))
        self.output_format_example = scrolledtext.ScrolledText(right_frame, height=6, width=45, 
                                                              state='disabled', wrap=tk.WORD,
                                                              font=('Consolas', 9))
        self.output_format_example.pack(fill=tk.BOTH, expand=True)
        
        # 初始化显示默认提示
        self.show_format_example("", self.input_format_example)
        self.show_format_example("", self.output_format_example)
    
    def create_input_selection_step(self, parent, row):
        """步骤2：输入选择"""
        step_frame = ttk.LabelFrame(parent, text=f"📂 {t('gui.step2_input')}", padding="20")
        step_frame.grid(row=row, column=0, sticky=(tk.W, tk.E), pady=(0, 20))
        step_frame.columnconfigure(0, weight=1)
        
        # 路径输入区域
        path_frame = ttk.Frame(step_frame)
        path_frame.grid(row=0, column=0, sticky=(tk.W, tk.E), pady=(0, 15))
        path_frame.columnconfigure(0, weight=1)
        
        self.input_var = tk.StringVar()
        input_entry = ttk.Entry(path_frame, textvariable=self.input_var, font=('TkDefaultFont', 10))
        input_entry.grid(row=0, column=0, sticky=(tk.W, tk.E), padx=(0, 15))
        
        # 按钮区域
        button_frame = ttk.Frame(path_frame)
        button_frame.grid(row=0, column=1)
        
        self.select_file_btn = ttk.Button(button_frame, text=t('gui.select_file'), 
                                        command=self.select_input_file, state='disabled',
                                        width=12)
        self.select_file_btn.grid(row=0, column=0, padx=(0, 8))
        
        self.select_folder_btn = ttk.Button(button_frame, text=t('gui.select_folder'), 
                                          command=self.select_input_folder, state='disabled',
                                          width=12)
        self.select_folder_btn.grid(row=0, column=1)
        
        # 提示信息
        self.input_hint = ttk.Label(step_frame, text=t('gui.select_format_first'), style='Info.TLabel')
        self.input_hint.grid(row=1, column=0, columnspan=2, sticky=tk.W)
    
    def create_output_selection_step(self, parent, row):
        """步骤3：输出选择"""
        step_frame = ttk.LabelFrame(parent, text=f"📁 {t('gui.step3_output')}", padding="20")
        step_frame.grid(row=row, column=0, sticky=(tk.W, tk.E), pady=(0, 20))
        step_frame.columnconfigure(0, weight=1)
        
        # 路径输入区域
        path_frame = ttk.Frame(step_frame)
        path_frame.grid(row=0, column=0, sticky=(tk.W, tk.E), pady=(0, 15))
        path_frame.columnconfigure(0, weight=1)
        
        self.output_var = tk.StringVar()
        output_entry = ttk.Entry(path_frame, textvariable=self.output_var, font=('TkDefaultFont', 10))
        output_entry.grid(row=0, column=0, sticky=(tk.W, tk.E), padx=(0, 15))
        
        self.select_output_btn = ttk.Button(path_frame, text=t('gui.select_folder'), 
                                          command=self.select_output_folder, state='disabled',
                                          width=12)
        self.select_output_btn.grid(row=0, column=1)
        
        # 提示信息
        self.output_hint = ttk.Label(step_frame, text=t('gui.select_output_format_first'), style='Info.TLabel')
        self.output_hint.grid(row=1, column=0, columnspan=2, sticky=tk.W)
    
    def create_dimensions_step(self, parent, row):
        """步骤4：图片尺寸"""
        step_frame = ttk.LabelFrame(parent, text=f"📐 {t('gui.step4_dimensions')}", padding="20")
        step_frame.grid(row=row, column=0, sticky=(tk.W, tk.E), pady=(0, 20))
        
        # 尺寸输入
        dims_frame = ttk.Frame(step_frame)
        dims_frame.grid(row=0, column=0, sticky=tk.W, pady=(0, 15))
        
        ttk.Label(dims_frame, text=t('gui.width'), style='Header.TLabel').grid(
            row=0, column=0, sticky=tk.W, padx=(0, 8))
        self.width_var = tk.StringVar(value=str(self.settings.last_image_width))
        width_entry = ttk.Entry(dims_frame, textvariable=self.width_var, width=12, font=('TkDefaultFont', 10))
        width_entry.grid(row=0, column=1, padx=(0, 25))
        
        ttk.Label(dims_frame, text=t('gui.height'), style='Header.TLabel').grid(
            row=0, column=2, sticky=tk.W, padx=(0, 8))
        self.height_var = tk.StringVar(value=str(self.settings.last_image_height))
        height_entry = ttk.Entry(dims_frame, textvariable=self.height_var, width=12, font=('TkDefaultFont', 10))
        height_entry.grid(row=0, column=3)
        
        # 快捷按钮
        presets_frame = ttk.Frame(step_frame)
        presets_frame.grid(row=1, column=0, sticky=tk.W)
        
        ttk.Label(presets_frame, text=t('gui.common_sizes'), style='Info.TLabel').pack(
            side=tk.LEFT, padx=(0, 15))
        
        ttk.Button(presets_frame, text="1920×1080", 
                  command=lambda: self.set_dimensions(1920, 1080), width=10).pack(side=tk.LEFT, padx=(0, 8))
        ttk.Button(presets_frame, text="1280×720", 
                  command=lambda: self.set_dimensions(1280, 720), width=10).pack(side=tk.LEFT, padx=(0, 8))
        ttk.Button(presets_frame, text="640×480", 
                  command=lambda: self.set_dimensions(640, 480), width=10).pack(side=tk.LEFT)
    
    def create_classes_confirmation_step(self, parent, row):
        """步骤5：类别确认"""
        step_frame = ttk.LabelFrame(parent, text=f"🏷️ {t('gui.step5_classes')}", padding="20")
        step_frame.grid(row=row, column=0, sticky=(tk.W, tk.E), pady=(0, 20))
        step_frame.columnconfigure(0, weight=1)
        
        # 类别列表显示
        classes_label = ttk.Label(step_frame, text="检测到的类别列表:", style='Header.TLabel')
        classes_label.grid(row=0, column=0, sticky=tk.W, pady=(0, 8))
        
        self.classes_text = scrolledtext.ScrolledText(step_frame, height=5, state='disabled', 
                                                     wrap=tk.WORD, font=('TkDefaultFont', 9))
        self.classes_text.grid(row=1, column=0, sticky=(tk.W, tk.E), pady=(0, 15))
        
        # 操作按钮
        btn_frame = ttk.Frame(step_frame)
        btn_frame.grid(row=2, column=0, sticky=tk.W, pady=(0, 10))
        
        self.load_classes_btn = ttk.Button(btn_frame, text=t('gui.load_classes_file'), 
                                         command=self.load_classes_file, width=16)
        self.load_classes_btn.pack(side=tk.LEFT, padx=(0, 12))
        
        self.refresh_classes_btn = ttk.Button(btn_frame, text=t('gui.refresh_classes'), 
                                            command=self.refresh_classes, state='disabled', width=12)
        self.refresh_classes_btn.pack(side=tk.LEFT)
        
        # 类别状态
        self.classes_status = ttk.Label(step_frame, text=t('gui.no_classes_detected'), style='Info.TLabel')
        self.classes_status.grid(row=3, column=0, sticky=tk.W)
    
    def create_conversion_step(self, parent, row):
        """转换步骤"""
        step_frame = ttk.LabelFrame(parent, text=f"🚀 {t('gui.step6_convert')}", padding="20")
        step_frame.grid(row=row, column=0, sticky=(tk.W, tk.E), pady=(0, 20))
        step_frame.columnconfigure(0, weight=1)
        
        # 转换摘要标题
        summary_label = ttk.Label(step_frame, text="转换配置摘要:", style='Header.TLabel')
        summary_label.grid(row=0, column=0, sticky=tk.W, pady=(0, 8))
        
        # 转换摘要
        summary_frame = ttk.Frame(step_frame, relief='sunken', borderwidth=1)
        summary_frame.grid(row=1, column=0, sticky=(tk.W, tk.E), pady=(0, 20))
        summary_frame.columnconfigure(0, weight=1)
        
        self.conversion_summary = ttk.Label(summary_frame, text="请完成以上步骤配置", 
                                           style='Info.TLabel', wraplength=600, 
                                           padding="15", justify='left')
        self.conversion_summary.grid(row=0, column=0, sticky=(tk.W, tk.E))
        
        # 转换按钮
        button_frame = ttk.Frame(step_frame)
        button_frame.grid(row=2, column=0)
        
        self.convert_button = ttk.Button(button_frame, text=t('gui.start_conversion'), 
                                       command=self.start_conversion, state='disabled',
                                       style='Primary.TButton', width=20, padding=(10, 8))
        self.convert_button.pack()
    
    def create_language_section(self, parent):
        """创建语言选择区域"""
        frame = ttk.LabelFrame(parent, text=t('gui.language'), padding="15")
        frame.pack(fill=tk.X, pady=(0, 15))
        
        self.language_var = tk.StringVar()
        languages = get_available_languages()
        
        language_combo = ttk.Combobox(frame, textvariable=self.language_var, 
                                    values=list(languages.values()), 
                                    state="readonly", width=20)
        language_combo.pack(anchor=tk.W)
        language_combo.bind('<<ComboboxSelected>>', self.on_language_change)
        
        # 设置当前语言
        current_lang = languages.get(self.settings.language, "English")
        self.language_var.set(current_lang)
        
        # 说明文字
        ttk.Label(frame, text=t('gui.language_restart_note'), style='Info.TLabel').pack(
            anchor=tk.W, pady=(5, 0))
    
    def create_author_section(self, parent):
        """创建作者信息区域"""
        frame = ttk.LabelFrame(parent, text="关于作者", padding="15")
        frame.pack(fill=tk.X, pady=(15, 0))
        
        # 作者名称
        author_frame = ttk.Frame(frame)
        author_frame.pack(fill=tk.X, pady=(0, 8))
        
        ttk.Label(author_frame, text="作者:", style='Header.TLabel').pack(side=tk.LEFT)
        ttk.Label(author_frame, text="BIANG", style='Info.TLabel', font=('TkDefaultFont', 12, 'bold')).pack(side=tk.LEFT, padx=(10, 0))
        
        # 项目信息
        project_frame = ttk.Frame(frame)
        project_frame.pack(fill=tk.X, pady=(0, 8))
        
        ttk.Label(project_frame, text="项目:", style='Header.TLabel').pack(side=tk.LEFT)
        ttk.Label(project_frame, text="OBB数据格式转换器", style='Info.TLabel').pack(side=tk.LEFT, padx=(10, 0))
        
        # 版本信息
        version_frame = ttk.Frame(frame)
        version_frame.pack(fill=tk.X, pady=(0, 8))
        
        ttk.Label(version_frame, text="版本:", style='Header.TLabel').pack(side=tk.LEFT)
        ttk.Label(version_frame, text="v1.0.0", style='Info.TLabel').pack(side=tk.LEFT, padx=(10, 0))
        
        # GitHub链接
        link_frame = ttk.Frame(frame)
        link_frame.pack(fill=tk.X)
        
        ttk.Label(link_frame, text="GitHub:", style='Header.TLabel').pack(side=tk.LEFT)
        ttk.Label(link_frame, text="github.com/BIANG-qilie/dataset-format-converter", 
                 style='Info.TLabel', foreground='#0066cc').pack(side=tk.LEFT, padx=(10, 0))
    
    def load_settings(self):
        """加载设置到界面"""
        # 加载输入格式设置
        if hasattr(self, 'input_format_var') and hasattr(self, 'input_format_example'):
            input_format = self.settings.last_input_format or ""
            self.input_format_var.set(input_format)
            # 更新示例显示
            self.show_format_example(input_format, self.input_format_example)
            # 如果有格式选择，触发格式变化事件以更新界面状态
            if input_format:
                self.on_input_format_change()
                
        # 加载输出格式设置        
        if hasattr(self, 'output_format_var') and hasattr(self, 'output_format_example'):
            output_format = self.settings.last_output_format or ""
            self.output_format_var.set(output_format)
            # 更新示例显示
            self.show_format_example(output_format, self.output_format_example)
            # 如果有格式选择，触发格式变化事件以更新界面状态
            if output_format:
                self.on_output_format_change()
        
        # 最后初始化界面状态
        if hasattr(self, 'conversion_summary'):
            self.update_conversion_summary()
    
    # 新的事件处理方法
    def on_input_format_change(self, event=None):
        """输入格式变化事件"""
        format_name = self.input_format_var.get()
        if format_name:
            # 启用输入选择按钮
            self.select_file_btn.config(state='normal')
            self.select_folder_btn.config(state='normal')
            self.input_hint.config(text=t('gui.format_selected_input_ready'))
            
            # 显示格式示例
            self.show_format_example(format_name, self.input_format_example)
            
            # 刷新类别
            if self.input_var.get():
                self.refresh_classes()
        
        self.update_conversion_summary()
    
    def on_output_format_change(self, event=None):
        """输出格式变化事件"""
        format_name = self.output_format_var.get()
        if format_name:
            # 启用输出选择按钮
            self.select_output_btn.config(state='normal')
            self.output_hint.config(text=t('gui.format_selected_output_ready'))
            
            # 显示格式示例
            self.show_format_example(format_name, self.output_format_example)
        
        self.update_conversion_summary()
    
    def show_format_example(self, format_name, text_widget):
        """显示格式示例"""
        examples = {
            '': '请选择一个格式以查看示例\n\n支持的格式包括：\n• YOLO-HBB - 水平边界框格式\n• YOLO-OBB - 旋转边界框格式\n• LabelImg-OBB - LabelImg工具格式\n• DOTA - 遥感数据格式\n• PASCAL-VOC - XML标注格式',
            'YOLO-HBB': '格式说明：\nclass_id x_center y_center width height\n(归一化坐标，范围0-1)\n\n示例内容：\n0 0.5 0.5 0.3 0.4\n1 0.2 0.3 0.1 0.2\n\n说明：\n第一行：类别0，中心点(0.5,0.5)，宽0.3，高0.4\n第二行：类别1，中心点(0.2,0.3)，宽0.1，高0.2',
            'YOLO-OBB': '格式说明：\nclass_id x1 y1 x2 y2 x3 y3 x4 y4\n(归一化坐标，四个角点)\n\n示例内容：\n0 0.1 0.1 0.9 0.1 0.9 0.9 0.1 0.9\n1 0.2 0.2 0.8 0.2 0.8 0.8 0.2 0.8\n\n说明：\n每行8个坐标值表示矩形的4个角点\n按顺序：左上→右上→右下→左下',
            'LabelImg-OBB': '格式说明：\n第一行固定为"YOLO_OBB"\n后续：class_id x_center y_center width height angle\n(像素坐标+角度)\n\n示例内容：\nYOLO_OBB\n0 960 540 800 600 45.0\n1 480 270 400 300 0.0\n\n说明：\n包含旋转角度信息的边界框格式',
            'DOTA': '格式说明：\nx1 y1 x2 y2 x3 y3 x4 y4 class_name [difficulty]\n(像素坐标+类别名+难度)\n\n示例内容：\n100 100 200 100 200 200 100 200 plane 0\n300 300 400 300 400 400 300 400 ship 1\n\n说明：\n遥感图像标注格式，支持任意四边形\n最后的数字表示标注难度（可选）',
            'PASCAL-VOC': '格式说明：\nXML文件格式，包含边界框信息\n\n示例结构：\n<?xml version="1.0"?>\n<annotation>\n  <object>\n    <name>person</name>\n    <bndbox>\n      <xmin>100</xmin>\n      <ymin>100</ymin>\n      <xmax>200</xmax>\n      <ymax>200</ymax>\n    </bndbox>\n  </object>\n</annotation>\n\n说明：标准的目标检测XML格式'
        }
        
        example_text = examples.get(format_name, f'{format_name} 格式示例')
        
        text_widget.config(state='normal')
        text_widget.delete(1.0, tk.END)
        text_widget.insert(1.0, example_text)
        text_widget.config(state='disabled')
    
    def select_input_file(self):
        """选择输入文件"""
        input_format = self.input_format_var.get()
        if not input_format:
            return
        
        # 根据格式设置文件类型过滤
        if input_format == 'PASCAL-VOC':
            filetypes = [(t('gui.xml_files'), "*.xml"), (t('gui.all_files'), "*.*")]
        else:
            filetypes = [(t('gui.text_files'), "*.txt"), (t('gui.all_files'), "*.*")]
        
        filename = filedialog.askopenfilename(
            title=t('gui.select_input_file'),
            filetypes=filetypes
        )
        if filename:
            self.input_var.set(filename)
            # 自动检测格式（可选）
            detected = format_manager.detect_format(filename)
            if detected and detected != input_format:
                result = messagebox.askyesno(
                    t('gui.format_mismatch_title'), 
                    t('gui.format_mismatch_message').format(
                        detected=detected, selected=input_format
                    )
                )
                if result:
                    self.input_format_var.set(detected)
                    self.on_input_format_change()
            
            # 刷新类别
            self.refresh_classes()
    
    def select_input_folder(self):
        """选择输入目录"""
        input_format = self.input_format_var.get()
        if not input_format:
            return
        
        dirname = filedialog.askdirectory(title=t('gui.select_input_folder'))
        if dirname:
            self.input_var.set(dirname)
            self.refresh_classes()
    
    def select_output_folder(self):
        """选择输出目录"""
        if not self.output_format_var.get():
            return
        
        dirname = filedialog.askdirectory(title=t('gui.select_output_folder'))
        if dirname:
            self.output_var.set(dirname)
            self.update_conversion_summary()
    
    def load_classes_file(self):
        """加载类别文件"""
        filename = filedialog.askopenfilename(
            title=t('gui.select_classes_file'),
            filetypes=[
                ("classes.txt", "classes.txt"),
                (t('gui.text_files'), "*.txt"),
                (t('gui.all_files'), "*.*")
            ]
        )
        if filename:
            try:
                with open(filename, 'r', encoding='utf-8') as f:
                    self.current_class_names = [line.strip() for line in f.readlines() if line.strip()]
                self.update_classes_display()
            except Exception as e:
                messagebox.showerror(t('gui.error'), f"{t('gui.load_classes_error')}: {e}")
    
    def refresh_classes(self):
        """刷新类别列表"""
        input_path = self.input_var.get()
        input_format = self.input_format_var.get()
        
        if not input_path or not input_format:
            self.current_class_names = []
            self.update_classes_display()
            return
        
        try:
            format_instance = format_manager.get_format(input_format)
            
            if os.path.isfile(input_path):
                self.current_class_names = format_instance._get_class_names([input_path])
            else:
                # 目录情况
                file_paths = []
                for f in os.listdir(input_path):
                    if f.endswith(format_instance.file_extension):
                        file_paths.append(os.path.join(input_path, f))
                self.current_class_names = format_instance._get_class_names(file_paths[:5])  # 只检查前5个文件
            
            self.update_classes_display()
            self.refresh_classes_btn.config(state='normal')
            
        except Exception as e:
            self.current_class_names = []
            self.update_classes_display()
            print(f"刷新类别时出错: {e}")
    
    def update_classes_display(self):
        """更新类别显示"""
        self.classes_text.config(state='normal')
        self.classes_text.delete(1.0, tk.END)
        
        if self.current_class_names:
            class_text = f"{t('gui.detected_classes')} ({len(self.current_class_names)}):\n\n"
            for i, class_name in enumerate(self.current_class_names):
                class_text += f"{i}: {class_name}\n"
            self.classes_text.insert(1.0, class_text)
            self.classes_status.config(text=f"{t('gui.classes_count')}: {len(self.current_class_names)}", 
                                     style='Success.TLabel')
        else:
            self.classes_text.insert(1.0, t('gui.no_classes_detected'))
            self.classes_status.config(text=t('gui.no_classes_detected'), style='Info.TLabel')
        
        self.classes_text.config(state='disabled')
        self.update_conversion_summary()
    
    def set_dimensions(self, width, height):
        """设置图片尺寸"""
        self.width_var.set(str(width))
        self.height_var.set(str(height))
        self.update_conversion_summary()
    
    def update_conversion_summary(self):
        """更新转换摘要"""
        if not hasattr(self, 'conversion_summary'):
            return
            
        input_format = self.input_format_var.get() if hasattr(self, 'input_format_var') else ""
        output_format = self.output_format_var.get() if hasattr(self, 'output_format_var') else ""
        input_path = self.input_var.get() if hasattr(self, 'input_var') else ""
        output_path = self.output_var.get() if hasattr(self, 'output_var') else ""
        
        if not all([input_format, output_format, input_path, output_path]):
            self.conversion_summary.config(text=t('gui.complete_all_steps'))
            if hasattr(self, 'convert_button'):
                self.convert_button.config(state='disabled')
            return
        
        try:
            width = int(self.width_var.get()) if hasattr(self, 'width_var') and self.width_var.get() else 0
            height = int(self.height_var.get()) if hasattr(self, 'height_var') and self.height_var.get() else 0
            if width <= 0 or height <= 0:
                raise ValueError
        except ValueError:
            self.conversion_summary.config(text=t('gui.invalid_dimensions'))
            if hasattr(self, 'convert_button'):
                self.convert_button.config(state='disabled')
            return
        
        # 创建摘要文本
        summary_parts = [
            f"{t('gui.input_format')}: {input_format}",
            f"{t('gui.output_format')}: {output_format}",
            f"{t('gui.input_path')}: {os.path.basename(input_path)}",
            f"{t('gui.output_path')}: {os.path.basename(output_path)}",
            f"{t('gui.dimensions')}: {width}×{height}",
        ]
        
        if hasattr(self, 'current_class_names') and self.current_class_names:
            summary_parts.append(f"{t('gui.classes_count')}: {len(self.current_class_names)}")
        
        summary_text = "\n".join(summary_parts)
        self.conversion_summary.config(text=summary_text)
        if hasattr(self, 'convert_button'):
            self.convert_button.config(state='normal')
    
    def on_language_change(self, event=None):
        """语言切换事件处理"""
        languages = get_available_languages()
        selected = self.language_var.get()
        
        for code, name in languages.items():
            if name == selected:
                set_language(code)
                update_settings(language=code)
                messagebox.showinfo(
                    "语言已更改" if code == 'zh' else "Language Changed", 
                    "请重启应用以完全应用新语言设置。" if code == 'zh' else "Please restart the application to fully apply the new language settings."
                )
                break
    
    def start_conversion(self):
        """开始转换（在新线程中执行）"""
        input_format = self.input_format_var.get()
        output_format = self.output_format_var.get()
        input_path = self.input_var.get()
        output_path = self.output_var.get()
        
        if not all([input_format, output_format, input_path, output_path]):
            messagebox.showerror(t('gui.error'), t('gui.complete_all_steps'))
            return
        
        try:
            width = int(self.width_var.get())
            height = int(self.height_var.get())
            if width <= 0 or height <= 0:
                raise ValueError
        except ValueError:
            messagebox.showerror(t('gui.error'), t('gui.invalid_dimensions'))
            return
        
        if not os.path.exists(input_path):
            messagebox.showerror(t('gui.error'), t('gui.input_not_exists'))
            return
        
        # 禁用转换按钮
        self.convert_button.config(state='disabled')
        self.progress_var.set(0)
        self.status_var.set(t('gui.converting'))
        
        # 在新线程中执行转换
        thread = threading.Thread(target=self.perform_conversion)
        thread.daemon = True
        thread.start()
    
    def perform_conversion(self):
        """执行转换操作"""
        try:
            input_path = self.input_var.get()
            output_path = self.output_var.get()
            input_format = self.input_format_var.get()
            output_format = self.output_format_var.get()
            width = int(self.width_var.get())
            height = int(self.height_var.get())
            
            # 更新进度
            self.root.after(0, lambda: self.progress_var.set(25))
            
            # 执行转换
            if os.path.isfile(input_path):
                # 单文件转换
                format_manager.convert_file(
                    input_file=input_path,
                    output_file=os.path.join(output_path, os.path.basename(input_path)),
                    input_format=input_format,
                    output_format=output_format,
                    image_width=width,
                    image_height=height,
                    class_names=self.current_class_names if self.current_class_names else None
                )
            else:
                # 目录转换
                format_manager.convert_directory(
                    input_dir=input_path,
                    output_dir=output_path,
                    input_format=input_format,
                    output_format=output_format,
                    image_width=width,
                    image_height=height,
                    class_names=self.current_class_names if self.current_class_names else None
                )
            
            # 更新进度
            self.root.after(0, lambda: self.progress_var.set(100))
            self.root.after(0, self.conversion_completed)
            
        except Exception as e:
            error_msg = str(e)
            self.root.after(0, lambda: self.conversion_failed(error_msg))
    
    def conversion_completed(self):
        """转换完成处理"""
        self.status_var.set(t('gui.conversion_completed'))
        self.convert_button.config(state='normal')
        self.progress_var.set(0)
        messagebox.showinfo(t('gui.success'), t('gui.conversion_completed'))
        
        # 保存设置
        update_settings(
            last_input_format=self.input_format_var.get(),
            last_output_format=self.output_format_var.get(),
            last_image_width=int(self.width_var.get()),
            last_image_height=int(self.height_var.get())
        )
    
    def conversion_failed(self, error_msg):
        """转换失败处理"""
        self.status_var.set(t('gui.conversion_failed'))
        self.convert_button.config(state='normal')
        self.progress_var.set(0)
        messagebox.showerror(t('gui.error'), f"{t('gui.conversion_failed')}: {error_msg}")
    
    def on_closing(self):
        """窗口关闭事件"""
        # 保存窗口大小
        width = self.root.winfo_width()
        height = self.root.winfo_height()
        update_settings(window_width=width, window_height=height)
        save_settings()
        self.root.destroy()
    
    def run(self):
        """运行GUI"""
        self.root.mainloop()


def main():
    """GUI主函数"""
    if not GUI_AVAILABLE:
        print("错误：tkinter模块不可用，无法启动图形界面")
        print("请安装完整的Python环境或安装tkinter模块")
        sys.exit(1)
    
    try:
        app = DatasetConverterGUI()
        app.run()
    except Exception as e:
        print(f"启动GUI失败: {e}")
        sys.exit(1)


if __name__ == "__main__":
    main() 


"""
PASCAL VOC 格式处理类

格式说明：
- XML格式文件
- 使用 <xmin>, <ymin>, <xmax>, <ymax> 标签
- 坐标为像素值
- 这是水平边界框格式（不支持旋转）
"""

import os
import xml.etree.ElementTree as ET
import numpy as np
from typing import List, Optional

from ..core.base_format import BaseFormat
from ..core.common_format import CommonFormat, BoundingBox
from ..core.geometry_utils import rect_to_corners, corners_to_rect, normalize_coordinates, denormalize_coordinates


class PascalVOCFormat(BaseFormat):
    """PASCAL VOC 格式处理类"""
    
    @property
    def name(self) -> str:
        return "PASCAL-VOC"
    
    @property
    def file_extension(self) -> str:
        return ".xml"
    
    @property
    def description(self) -> str:
        return "PASCAL VOC format: XML with <xmin>, <ymin>, <xmax>, <ymax> tags (pixel coordinates)"
    
    def verify(self, file_path: str) -> bool:
        """
        验证文件是否符合PASCAL VOC格式
        
        Args:
            file_path: 文件路径
            
        Returns:
            bool: 是否符合格式
        """
        if not os.path.exists(file_path):
            return False
        
        if not file_path.endswith('.xml'):
            return False
        
        try:
            tree = ET.parse(file_path)
            root = tree.getroot()
            
            # 检查根节点是否为annotation
            if root.tag != 'annotation':
                return False
            
            # 检查是否有object节点
            objects = root.findall('object')
            
            for obj in objects:
                # 检查必需的子节点
                name = obj.find('name')
                bndbox = obj.find('bndbox')
                
                if name is None or bndbox is None:
                    return False
                
                # 检查边界框坐标
                xmin = bndbox.find('xmin')
                ymin = bndbox.find('ymin')
                xmax = bndbox.find('xmax')
                ymax = bndbox.find('ymax')
                
                if any(coord is None for coord in [xmin, ymin, xmax, ymax]):
                    return False
                
                # 检查坐标是否为数字
                try:
                    float(xmin.text)
                    float(ymin.text)
                    float(xmax.text)
                    float(ymax.text)
                except (ValueError, TypeError):
                    return False
            
            return True
            
        except ET.ParseError:
            return False
        except Exception:
            return False
    
    def _format2common(self, file_path: str, image_width: int, image_height: int,
                      class_names: Optional[List[str]] = None) -> CommonFormat:
        """
        将PASCAL VOC格式转换为中间格式
        
        Args:
            file_path: 输入文件路径
            image_width: 图片宽度
            image_height: 图片高度
            class_names: 类别名称列表（将被更新）
            
        Returns:
            CommonFormat: 中间格式对象
        """
        if class_names is None:
            class_names = []
        
        bounding_boxes = []
        
        tree = ET.parse(file_path)
        root = tree.getroot()
        
        # 提取图片信息（如果XML中有的话）
        size_node = root.find('size')
        if size_node is not None:
            width_node = size_node.find('width')
            height_node = size_node.find('height')
            if width_node is not None and height_node is not None:
                try:
                    xml_width = int(width_node.text)
                    xml_height = int(height_node.text)
                    # 如果XML中的尺寸与提供的尺寸不同，使用XML中的
                    if xml_width != image_width or xml_height != image_height:
                        print(f"警告：XML中的图片尺寸 ({xml_width}x{xml_height}) 与提供的尺寸 ({image_width}x{image_height}) 不一致，使用XML中的尺寸")
                        image_width = xml_width
                        image_height = xml_height
                except ValueError:
                    pass
        
        # 处理对象
        objects = root.findall('object')
        
        for obj in objects:
            name_node = obj.find('name')
            bndbox = obj.find('bndbox')
            
            if name_node is None or bndbox is None:
                continue
            
            class_name = name_node.text
            
            # 提取边界框坐标
            xmin = float(bndbox.find('xmin').text)
            ymin = float(bndbox.find('ymin').text)
            xmax = float(bndbox.find('xmax').text)
            ymax = float(bndbox.find('ymax').text)
            
            # 转换为四个角点（像素坐标）
            pixel_corners = rect_to_corners(xmin, ymin, xmax, ymax)
            
            # 转换为归一化坐标
            normalized_corners = normalize_coordinates(pixel_corners, image_width, image_height)
            
            # 更新类别名称列表
            if class_name not in class_names:
                class_names.append(class_name)
            
            # 创建边界框对象
            bbox = BoundingBox(
                class_name=class_name,
                corners=normalized_corners,
                class_id=class_names.index(class_name)
            )
            
            bounding_boxes.append(bbox)
        
        return CommonFormat(
            image_width=image_width,
            image_height=image_height,
            bounding_boxes=bounding_boxes,
            class_names=class_names,
            image_filename=os.path.splitext(os.path.basename(file_path))[0]
        )
    
    def _common2format(self, common_data: CommonFormat, output_path: str) -> None:
        """
        将中间格式转换为PASCAL VOC格式
        
        Args:
            common_data: 中间格式数据
            output_path: 输出文件路径
        """
        # 创建根节点
        root = ET.Element('annotation')
        
        # 添加文件名（如果有的话）
        if common_data.image_filename:
            filename = ET.SubElement(root, 'filename')
            filename.text = f"{common_data.image_filename}.jpg"  # 假设是jpg格式
        
        # 添加图片尺寸信息
        size = ET.SubElement(root, 'size')
        width = ET.SubElement(size, 'width')
        height = ET.SubElement(size, 'height')
        depth = ET.SubElement(size, 'depth')
        
        width.text = str(common_data.image_width)
        height.text = str(common_data.image_height)
        depth.text = '3'  # 假设是RGB图像
        
        # 添加分割信息
        segmented = ET.SubElement(root, 'segmented')
        segmented.text = '0'
        
        # 处理每个边界框
        for bbox in common_data.bounding_boxes:
            # 将归一化坐标转换为像素坐标
            pixel_corners = denormalize_coordinates(
                bbox.corners,
                common_data.image_width,
                common_data.image_height
            )
            
            # 转换为矩形边界框（PASCAL VOC只支持水平边界框）
            xmin, ymin, xmax, ymax = corners_to_rect(pixel_corners)
            
            # 创建object节点
            obj = ET.SubElement(root, 'object')
            
            # 添加类别名称
            name = ET.SubElement(obj, 'name')
            name.text = bbox.class_name
            
            # 添加姿态信息
            pose = ET.SubElement(obj, 'pose')
            pose.text = 'Unspecified'
            
            # 添加截断信息
            truncated = ET.SubElement(obj, 'truncated')
            truncated.text = '0'
            
            # 添加遮挡信息
            occluded = ET.SubElement(obj, 'occluded')
            occluded.text = '0'
            
            # 添加边界框
            bndbox = ET.SubElement(obj, 'bndbox')
            xmin_elem = ET.SubElement(bndbox, 'xmin')
            ymin_elem = ET.SubElement(bndbox, 'ymin')
            xmax_elem = ET.SubElement(bndbox, 'xmax')
            ymax_elem = ET.SubElement(bndbox, 'ymax')
            
            xmin_elem.text = f"{xmin:.0f}"
            ymin_elem.text = f"{ymin:.0f}"
            xmax_elem.text = f"{xmax:.0f}"
            ymax_elem.text = f"{ymax:.0f}"
            
            # 添加难度信息（如果有的话）
            difficult = ET.SubElement(obj, 'difficult')
            if bbox.difficulty is not None:
                difficult.text = str(bbox.difficulty)
            else:
                difficult.text = '0'
        
        # 写入文件
        tree = ET.ElementTree(root)
        ET.indent(tree, space="  ", level=0)  # 格式化XML
        tree.write(output_path, encoding='utf-8', xml_declaration=True) 

    def _extract_class_names_from_files(self, file_paths: List[str]) -> List[str]:
        """
        从PASCAL VOC格式文件中提取类别名称
        
        Args:
            file_paths: 文件路径列表
            
        Returns:
            List[str]: 类别名称列表
        """
        class_names = set()
        
        for file_path in file_paths:
            if not self.verify(file_path):
                continue
                
            try:
                tree = ET.parse(file_path)
                root = tree.getroot()
                objects = root.findall('object')
                
                for obj in objects:
                    name = obj.find('name')
                    if name is not None and name.text:
                        class_names.add(name.text)
            except Exception as e:
                print(f"警告：读取文件 {file_path} 时出错: {e}")
                
        return list(class_names)
    


"""
翻译管理器 - 处理多语言支持
"""

import os
import json
from typing import Dict, Any


class Translation:
    """翻译管理器"""
    
    def __init__(self):
        """初始化翻译管理器"""
        self.current_lang = 'en'
        self.translations: Dict[str, Dict[str, Any]] = {}
        self._load_translations()
    
    def _load_translations(self) -> None:
        """加载翻译文件"""
        # 获取当前文件所在目录
        current_dir = os.path.dirname(os.path.abspath(__file__))
        locales_dir = os.path.join(current_dir, 'locales')
        
        if not os.path.exists(locales_dir):
            os.makedirs(locales_dir)
            self._create_default_translations(locales_dir)
        
        # 加载所有语言文件
        for filename in os.listdir(locales_dir):
            if filename.endswith('.json'):
                lang_code = filename[:-5]  # 去掉.json扩展名
                file_path = os.path.join(locales_dir, filename)
                
                try:
                    with open(file_path, 'r', encoding='utf-8') as f:
                        self.translations[lang_code] = json.load(f)
                except Exception as e:
                    print(f"警告：无法加载语言文件 {filename}: {e}")
    
    def _create_default_translations(self, locales_dir: str) -> None:
        """创建默认的翻译文件"""
        # 英文翻译
        en_translations = {
            "app": {
                        "title": "Dataset Format Converter",
        "description": "Multi-format dataset annotation converter"
            },
            "formats": {
                "yolo_hbb": "YOLO-HBB",
                "yolo_obb": "YOLO-OBB",
                "labelimg_obb": "LabelImg-OBB",
                "dota": "DOTA",
                "pascal_voc": "PASCAL VOC"
            },
            "gui": {
                "input_format": "Input Format:",
                "output_format": "Output Format:",
                "input_path": "Input Path:",
                "output_path": "Output Path:",
                "image_width": "Image Width:",
                "image_height": "Image Height:",
                "class_names": "Class Names:",
                "browse": "Browse",
                "convert": "Convert",
                "cancel": "Cancel",
                "success": "Success",
                "error": "Error",
                "warning": "Warning",
                "info": "Info",
                "file": "File",
                "directory": "Directory",
                "language": "Language",
                "settings": "Settings"
            },
            "cli": {
                "usage": "Usage",
                "options": "Options",
                "input": "Input file or directory",
                "output": "Output file or directory",
                "input_format": "Input format",
                "output_format": "Output format",
                "width": "Image width",
                "height": "Image height",
                "classes": "Class names file",
                "help": "Show this help message",
                "version": "Show version information"
            },
            "messages": {
                "conversion_complete": "Conversion completed successfully!",
                "conversion_failed": "Conversion failed: {error}",
                "file_not_found": "File not found: {file}",
                "invalid_format": "Invalid format: {format}",
                "invalid_dimensions": "Invalid image dimensions",
                "no_files_found": "No files found in directory",
                "creating_output_dir": "Creating output directory: {dir}",
                "processing_file": "Processing file: {file}",
                "skipping_file": "Skipping file: {file}"
            }
        }
        
        # 中文翻译
        zh_translations = {
            "app": {
                "title": "OBB数据格式转换器",
                "description": "多格式OBB标注转换工具"
            },
            "formats": {
                "yolo_hbb": "YOLO-HBB",
                "yolo_obb": "YOLO-OBB",
                "labelimg_obb": "LabelImg-OBB",
                "dota": "DOTA",
                "pascal_voc": "PASCAL VOC"
            },
            "gui": {
                "input_format": "输入格式：",
                "output_format": "输出格式：",
                "input_path": "输入路径：",
                "output_path": "输出路径：",
                "image_width": "图片宽度：",
                "image_height": "图片高度：",
                "class_names": "类别名称：",
                "browse": "浏览",
                "convert": "转换",
                "cancel": "取消",
                "success": "成功",
                "error": "错误",
                "warning": "警告",
                "info": "信息",
                "file": "文件",
                "directory": "目录",
                "language": "语言",
                "settings": "设置"
            },
            "cli": {
                "usage": "用法",
                "options": "选项",
                "input": "输入文件或目录",
                "output": "输出文件或目录",
                "input_format": "输入格式",
                "output_format": "输出格式",
                "width": "图片宽度",
                "height": "图片高度",
                "classes": "类别名称文件",
                "help": "显示帮助信息",
                "version": "显示版本信息"
            },
            "messages": {
                "conversion_complete": "转换成功完成！",
                "conversion_failed": "转换失败：{error}",
                "file_not_found": "文件未找到：{file}",
                "invalid_format": "无效格式：{format}",
                "invalid_dimensions": "无效的图片尺寸",
                "no_files_found": "目录中未找到文件",
                "creating_output_dir": "创建输出目录：{dir}",
                "processing_file": "处理文件：{file}",
                "skipping_file": "跳过文件：{file}"
            }
        }
        
        # 保存翻译文件
        with open(os.path.join(locales_dir, 'en.json'), 'w', encoding='utf-8') as f:
            json.dump(en_translations, f, indent=2, ensure_ascii=False)
        
        with open(os.path.join(locales_dir, 'zh.json'), 'w', encoding='utf-8') as f:
            json.dump(zh_translations, f, indent=2, ensure_ascii=False)
    
    def set_language(self, lang_code: str) -> None:
        """设置当前语言"""
        if lang_code in self.translations:
            self.current_lang = lang_code
        else:
            print(f"警告：不支持的语言代码 '{lang_code}'，使用默认语言 'en'")
            self.current_lang = 'en'
    
    def translate(self, key: str, **kwargs) -> str:
        """
        翻译文本
        
        Args:
            key: 翻译键，使用点号分隔嵌套结构，如 'gui.input_format'
            **kwargs: 格式化参数
            
        Returns:
            str: 翻译后的文本
        """
        # 获取当前语言的翻译
        current_translations = self.translations.get(self.current_lang, {})
        
        # 按点号分割键
        keys = key.split('.')
        value = current_translations
        
        for k in keys:
            if isinstance(value, dict) and k in value:
                value = value[k]
            else:
                # 如果在当前语言中找不到，尝试使用英文
                if self.current_lang != 'en':
                    en_translations = self.translations.get('en', {})
                    value = en_translations
                    for k in keys:
                        if isinstance(value, dict) and k in value:
                            value = value[k]
                        else:
                            # 如果英文中也找不到，返回键本身
                            return key
                else:
                    return key
                break
        
        # 如果最终值不是字符串，返回键本身
        if not isinstance(value, str):
            return key
        
        # 格式化字符串
        try:
            return value.format(**kwargs)
        except (KeyError, ValueError):
            return value
    
    def get_available_languages(self) -> Dict[str, str]:
        """获取可用的语言列表"""
        return {
            'en': 'English',
            'zh': '简体中文'
        }


# 全局翻译实例
_translation = Translation()


def t(key: str, **kwargs) -> str:
    """
    翻译函数的简化接口
    
    Args:
        key: 翻译键
        **kwargs: 格式化参数
        
    Returns:
        str: 翻译后的文本
    """
    return _translation.translate(key, **kwargs)


def set_language(lang_code: str) -> None:
    """设置语言"""
    _translation.set_language(lang_code)


def get_current_language() -> str:
    """获取当前语言"""
    return _translation.current_lang


def get_available_languages() -> Dict[str, str]:
    """获取可用语言列表"""
    return _translation.get_available_languages() 